@@ -94,9 +94,24 @@ typedef enum {
 	GS_APP_RATING_KIND_UNKNOWN,
 	GS_APP_RATING_KIND_USER,
 	GS_APP_RATING_KIND_SYSTEM,
+	GS_APP_RATING_KIND_KUDOS,
 	GS_APP_RATING_KIND_LAST
 } GsAppRatingKind;
 
+typedef enum {
+	GS_APP_KUDO_MY_LANGUAGE			= 1 << 0,
+	GS_APP_KUDO_RECENT_RELEASE		= 1 << 1,
+	GS_APP_KUDO_FEATURED_RECOMMENDED	= 1 << 2,
+	GS_APP_KUDO_MODERN_TOOLKIT		= 1 << 3,
+	GS_APP_KUDO_SEARCH_PROVIDER		= 1 << 4,
+	GS_APP_KUDO_INSTALLS_USER_DOCS		= 1 << 5,
+	GS_APP_KUDO_USES_NOTIFICATIONS		= 1 << 6,
+	GS_APP_KUDO_HAS_KEYWORDS		= 1 << 7,
+	GS_APP_KUDO_USES_APP_MENU		= 1 << 8,
+	GS_APP_KUDO_HAS_SCREENSHOTS		= 1 << 9,
+	GS_APP_KUDO_LAST
+} GsAppKudo;
+
 #define	GS_APP_INSTALL_DATE_UNSET		0
 #define	GS_APP_INSTALL_DATE_UNKNOWN		1 /* 1s past the epoch */
 #define	GS_APP_SIZE_UNKNOWN			0
@@ -113,6 +128,8 @@ typedef enum {
 	GS_APP_QUALITY_LAST
 } GsAppQuality;
 
+#define	GS_APP_KUDOS_WEIGHT_TO_PERCENTAGE(w)	(w * 20)
+
 GQuark		 gs_app_error_quark		(void);
 GType		 gs_app_get_type		(void);
 
@@ -246,6 +263,11 @@ void		 gs_app_add_category		(GsApp		*app,
 GPtrArray	*gs_app_get_keywords		(GsApp		*app);
 void		 gs_app_set_keywords		(GsApp		*app,
 						 GPtrArray	*keywords);
+void		 gs_app_add_kudo		(GsApp		*app,
+						 GsAppKudo	 kudo);
+guint64		 gs_app_get_kudos		(GsApp		*app);
+guint		 gs_app_get_kudos_weight	(GsApp		*app);
+guint		 gs_app_get_kudos_percentage	(GsApp		*app);
 
 G_END_DECLS
 
