@@ -81,11 +81,11 @@ gs_shell_details_set_state (GsShellDetails *shell_details,
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "spinner_details"));
 	switch (state) {
 	case GS_SHELL_DETAILS_STATE_LOADING:
-		gtk_spinner_start (GTK_SPINNER (widget));
+		gs_start_spinner (GTK_SPINNER (widget));
 		gtk_widget_show (widget);
 		break;
 	case GS_SHELL_DETAILS_STATE_READY:
-		gtk_spinner_stop (GTK_SPINNER (widget));
+		gs_stop_spinner (GTK_SPINNER (widget));
 		gtk_widget_hide (widget);
 		break;
 	default:
@@ -487,6 +487,21 @@ out:
 	g_strfreev (split);
 }
 
+/**
+ * gs_shell_details_is_addon_id_kind
+ **/
+static gboolean
+gs_shell_details_is_addon_id_kind (GsApp *app)
+{
+        GsAppIdKind id_kind;
+        id_kind = gs_app_get_id_kind (app);
+        if (id_kind == GS_APP_ID_KIND_DESKTOP)
+                return FALSE;
+        if (id_kind == GS_APP_ID_KIND_WEBAPP)
+                return FALSE;
+        return TRUE;
+}
+
 /**
  * gs_shell_details_refresh_all:
  **/
@@ -540,8 +555,14 @@ gs_shell_details_refresh_all (GsShellDetails *shell_details)
 	}
 	if (pixbuf == NULL)
 		pixbuf = gs_app_get_pixbuf (priv->app);
-	if( pixbuf == NULL && gs_app_get_state (priv->app) == GS_APP_STATE_LOCAL)
-		pixbuf = gs_pixbuf_load ("package-x-generic-symbolic", 96, NULL);
+	if (pixbuf == NULL && gs_app_get_state (priv->app) == GS_APP_STATE_LOCAL) {
+		if (gs_app_get_kind (priv->app) == GS_APP_KIND_SOURCE)
+			pixbuf = gs_pixbuf_load ("x-package-repository", 96, NULL);
+		else if (gs_shell_details_is_addon_id_kind (priv->app))
+			pixbuf = gs_pixbuf_load ("application-x-addon", 96, NULL);
+		else
+			pixbuf = gs_pixbuf_load ("application-x-executable", 96, NULL);
+	}
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "application_details_icon"));
 	if (pixbuf != NULL) {
 		gtk_image_set_from_pixbuf (GTK_IMAGE (widget), pixbuf);
@@ -655,11 +676,13 @@ gs_shell_details_refresh_all (GsShellDetails *shell_details)
 		gtk_label_set_label (GTK_LABEL (widget), tmp);
 	}
 	gtk_widget_set_visible (widget,
-				gs_app_get_state (priv->app) == GS_APP_STATE_INSTALLED);
+				gs_app_get_state (priv->app) == GS_APP_STATE_INSTALLED ||
+				gs_app_get_state (priv->app) == GS_APP_STATE_LOCAL);
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
 						     "label_details_origin_title"));
 	gtk_widget_set_visible (widget,
-				gs_app_get_state (priv->app) == GS_APP_STATE_INSTALLED);
+				gs_app_get_state (priv->app) == GS_APP_STATE_INSTALLED ||
+				gs_app_get_state (priv->app) == GS_APP_STATE_LOCAL);
 
 	/* set the rating */
 	switch (gs_app_get_id_kind (priv->app)) {
@@ -668,9 +691,15 @@ gs_shell_details_refresh_all (GsShellDetails *shell_details)
 		break;
 	default:
 		gtk_widget_set_visible (priv->star, TRUE);
-		gs_star_widget_set_rating (GS_STAR_WIDGET (priv->star),
-					   gs_app_get_rating_kind (priv->app),
-					   gs_app_get_rating (priv->app));
+		if (gs_app_get_rating_kind (priv->app) == GS_APP_RATING_KIND_USER) {
+			gs_star_widget_set_rating (GS_STAR_WIDGET (priv->star),
+						   GS_APP_RATING_KIND_USER,
+						   gs_app_get_rating (priv->app));
+		} else {
+			gs_star_widget_set_rating (GS_STAR_WIDGET (priv->star),
+						   GS_APP_RATING_KIND_KUDOS,
+						   gs_app_get_kudos_percentage (priv->app));
+		}
 		break;
 	}
 
@@ -702,17 +731,22 @@ gs_shell_details_refresh_all (GsShellDetails *shell_details)
 
 	/* make history button insensitive if there is none */
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_history"));
+	history = gs_app_get_history (priv->app);
 	switch (gs_app_get_id_kind (priv->app)) {
 	case GS_APP_ID_KIND_WEBAPP:
 		gtk_widget_set_visible (widget, FALSE);
 		break;
 	default:
-		history = gs_app_get_history (priv->app);
 		gtk_widget_set_sensitive (widget, history->len > 0);
 		gtk_widget_set_visible (widget, TRUE);
 		break;
 	}
 
+	/* don't show missing history on a local file */
+	if (gs_app_get_state (priv->app) == GS_APP_STATE_LOCAL &&
+	    history->len == 0)
+		gtk_widget_set_visible (widget, FALSE);
+
 	/* are we trying to replace something in the baseos */
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "infobar_details_package_baseos"));
 	switch (gs_app_get_kind (priv->app)) {
@@ -735,17 +769,6 @@ gs_shell_details_refresh_all (GsShellDetails *shell_details)
 		break;
 	}
 
-	/* an application */
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "infobar_details_repo"));
-	switch (gs_app_get_kind (priv->app)) {
-	case GS_APP_KIND_SOURCE:
-		gtk_widget_set_visible (widget, gs_app_get_state (priv->app) == GS_APP_STATE_LOCAL);
-		break;
-	default:
-		gtk_widget_set_visible (widget, FALSE);
-		break;
-	}
-
 	/* installing a app with a repo file */
 	tmp = gs_app_get_metadata_item (priv->app, "PackageKit::has-source");
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "infobar_details_app_repo"));
@@ -889,6 +912,7 @@ gs_shell_details_set_app (GsShellDetails *shell_details, GsApp *app)
 					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_HISTORY |
 					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_SETUP_ACTION |
 					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_ORIGIN |
+					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_MENU_PATH |
 					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_URL,
 					   priv->cancellable,
 					   gs_shell_details_app_refine_cb,
@@ -1219,17 +1243,6 @@ gs_shell_details_app_history_button_cb (GtkWidget *widget, GsShellDetails *shell
 	gtk_window_present (GTK_WINDOW (widget));
 }
 
-/**
- * gs_shell_details_button_close_cb:
- **/
-static void
-gs_shell_details_button_close_cb (GtkWidget *widget, GsShellDetails *shell_details)
-{
-	GsShellDetailsPrivate *priv = shell_details->priv;
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "dialog_history"));
-	gtk_widget_hide (widget);
-}
-
 /**
  * gs_shell_details_list_header_func
  **/
@@ -1385,10 +1398,6 @@ gs_shell_details_setup (GsShellDetails *shell_details,
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gs_shell_details_website_cb),
 			  shell_details);
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_history_close"));
-	g_signal_connect (widget, "clicked",
-			  G_CALLBACK (gs_shell_details_button_close_cb),
-			  shell_details);
 
 	/* setup history window */
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "dialog_history"));
