@@ -57,6 +57,8 @@ GsShellDetails	*gs_shell_details_new		(void);
 void		 gs_shell_details_invalidate	(GsShellDetails		*shell_details);
 void		 gs_shell_details_set_app	(GsShellDetails		*shell_details,
 						 GsApp			*app);
+void		 gs_shell_details_set_filename	(GsShellDetails		*shell_details,
+						 const gchar		*filename);
 GsApp		*gs_shell_details_get_app       (GsShellDetails		*shell_details);
 void		 gs_shell_details_refresh	(GsShellDetails		*shell_details);
 void		 gs_shell_details_setup		(GsShellDetails		*shell_details,
