@@ -90,7 +90,9 @@ gs_shell_search_app_installed_cb (GObject *source,
 					    error);
 		g_error_free (error);
 	} else {
-		gs_app_notify_installed (helper->app);
+		/* only show this if the window is not active */
+		if (!gs_shell_is_active (helper->shell_search->priv->shell))
+			gs_app_notify_installed (helper->app);
 	}
 	g_object_unref (helper->app);
 	g_object_unref (helper->shell_search);
@@ -257,7 +259,14 @@ gs_shell_search_get_search_cb (GObject *source_object,
 
 	list = gs_plugin_loader_search_finish (plugin_loader, res, &error);
 	if (list == NULL) {
-		g_warning ("failed to get search apps: %s", error->message);
+		if (g_error_matches (error,
+				     GS_PLUGIN_LOADER_ERROR,
+				     GS_PLUGIN_LOADER_ERROR_NO_RESULTS)) {
+			g_debug ("no search results to show");
+		} else {
+			g_warning ("failed to get search apps: %s",
+				   error->message);
+		}
 		g_error_free (error);
 		gtk_stack_set_visible_child_name (GTK_STACK (widget), "no-results");
 		goto out;
