@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2013 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2014 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -198,8 +198,10 @@ appstream_cache_start_element_cb (GMarkupParseContext *context,
 	case APPSTREAM_TAG_KEYWORDS:
 	case APPSTREAM_TAG_KEYWORD:
 	case APPSTREAM_TAG_MIMETYPES:
+	case APPSTREAM_TAG_METADATA:
 	case APPSTREAM_TAG_MIMETYPE:
 	case APPSTREAM_TAG_PRIORITY:
+	case APPSTREAM_TAG_LANGUAGES:
 		/* ignore */
 		break;
 	case APPSTREAM_TAG_ID:
@@ -260,6 +262,33 @@ appstream_cache_start_element_cb (GMarkupParseContext *context,
 			appstream_screenshot_set_kind (helper->screenshot, kind);
 		}
 		break;
+
+	case APPSTREAM_TAG_LANG:
+		if (helper->tag != APPSTREAM_TAG_LANGUAGES) {
+			g_set_error (error,
+				     APPSTREAM_CACHE_ERROR,
+				     APPSTREAM_CACHE_ERROR_FAILED,
+				     "XML start %s invalid, tag %s",
+				     element_name,
+				     appstream_tag_to_string (helper->tag));
+			return;
+		}
+
+		/* get the lang percentage */
+		for (i = 0; attribute_names[i] != NULL; i++) {
+			if (g_strcmp0 (attribute_names[i], "percentage") == 0) {
+				tmp = attribute_values[i];
+				break;
+			}
+		}
+
+		/* save language percentage */
+		if (tmp == NULL)
+			helper->priority = 100;
+		else
+			helper->priority = atoi (tmp);
+		break;
+
 	case APPSTREAM_TAG_CAPTION:
 		if (helper->screenshot == NULL ||
 		    helper->tag != APPSTREAM_TAG_SCREENSHOT) {
@@ -283,6 +312,29 @@ appstream_cache_start_element_cb (GMarkupParseContext *context,
 			return;
 		if (!helper->lang_temp)
 			helper->lang_temp = g_strdup ("C");
+		break;
+
+	case APPSTREAM_TAG_VALUE:
+		if (helper->tag != APPSTREAM_TAG_METADATA) {
+			g_set_error (error,
+				     APPSTREAM_CACHE_ERROR,
+				     APPSTREAM_CACHE_ERROR_FAILED,
+				     "XML start %s invalid, tag %s",
+				     element_name,
+				     appstream_tag_to_string (helper->tag));
+			return;
+		}
+
+		/* get lang */
+		if (!g_markup_collect_attributes (element_name,
+						  attribute_names,
+						  attribute_values,
+						  error,
+						  G_MARKUP_COLLECT_STRDUP,
+						  "key", &helper->lang_temp,
+						  G_MARKUP_COLLECT_INVALID))
+			return;
+		break;
 
 	case APPSTREAM_TAG_IMAGE:
 		if (helper->item_temp == NULL ||
@@ -384,7 +436,7 @@ appstream_cache_start_element_cb (GMarkupParseContext *context,
 			helper->url_type_temp = g_strdup ("homepage");
 		break;
 	case APPSTREAM_TAG_PKGNAME:
-	case APPSTREAM_TAG_LICENCE:
+	case APPSTREAM_TAG_PROJECT_LICENSE:
 	case APPSTREAM_TAG_PROJECT_GROUP:
 		if (helper->item_temp == NULL ||
 		    helper->tag != APPSTREAM_TAG_APPLICATION) {
@@ -541,7 +593,6 @@ appstream_cache_end_element_cb (GMarkupParseContext *context,
 	section_new = appstream_tag_from_string (element_name);
 	switch (section_new) {
 	case APPSTREAM_TAG_APPLICATIONS:
-	case APPSTREAM_TAG_APPCATEGORY:
 	case APPSTREAM_TAG_KEYWORD:
 	case APPSTREAM_TAG_MIMETYPE:
 		/* ignore */
@@ -552,6 +603,9 @@ appstream_cache_end_element_cb (GMarkupParseContext *context,
 		helper->item_temp = NULL;
 		helper->tag = APPSTREAM_TAG_APPLICATIONS;
 		break;
+	case APPSTREAM_TAG_APPCATEGORY:
+		helper->tag = APPSTREAM_TAG_APPCATEGORIES;
+		break;
 	case APPSTREAM_TAG_IMAGE:
 		appstream_screenshot_add_image (helper->screenshot,
 						helper->image);
@@ -564,13 +618,20 @@ appstream_cache_end_element_cb (GMarkupParseContext *context,
 		helper->screenshot = NULL;
 		helper->tag = APPSTREAM_TAG_SCREENSHOTS;
 		break;
+	case APPSTREAM_TAG_LANG:
+		helper->tag = APPSTREAM_TAG_LANGUAGES;
+		break;
+	case APPSTREAM_TAG_LANGUAGES:
+		helper->tag = APPSTREAM_TAG_APPLICATION;
+		break;
 	case APPSTREAM_TAG_ID:
 	case APPSTREAM_TAG_PKGNAME:
 	case APPSTREAM_TAG_APPCATEGORIES:
 	case APPSTREAM_TAG_COMPULSORY_FOR_DESKTOP:
 	case APPSTREAM_TAG_KEYWORDS:
 	case APPSTREAM_TAG_MIMETYPES:
-	case APPSTREAM_TAG_LICENCE:
+	case APPSTREAM_TAG_METADATA:
+	case APPSTREAM_TAG_PROJECT_LICENSE:
 	case APPSTREAM_TAG_ICON:
 		helper->tag = APPSTREAM_TAG_APPLICATION;
 		break;
@@ -591,6 +652,17 @@ appstream_cache_end_element_cb (GMarkupParseContext *context,
 	case APPSTREAM_TAG_PRIORITY:
 		helper->tag = APPSTREAM_TAG_APPLICATIONS;
 		break;
+	case APPSTREAM_TAG_VALUE:
+		/* a tag with no content, e.g. <value key=name/> */
+		if (helper->lang_temp != NULL) {
+			appstream_app_add_metadata (helper->item_temp,
+						    helper->lang_temp,
+						    "", 0);
+			g_free (helper->lang_temp);
+			helper->lang_temp = NULL;
+		}
+		helper->tag = APPSTREAM_TAG_METADATA;
+		break;
 	default:
 		/* ignore unknown entries */
 		helper->tag = helper->tag_last_known;
@@ -617,6 +689,7 @@ appstream_cache_text_cb (GMarkupParseContext *context,
 	case APPSTREAM_TAG_APPCATEGORIES:
 	case APPSTREAM_TAG_KEYWORDS:
 	case APPSTREAM_TAG_MIMETYPES:
+	case APPSTREAM_TAG_METADATA:
 		/* ignore */
 		break;
 	case APPSTREAM_TAG_PRIORITY:
@@ -659,6 +732,21 @@ appstream_cache_text_cb (GMarkupParseContext *context,
 		}
 		appstream_app_add_mimetype (helper->item_temp, text, text_len);
 		break;
+	case APPSTREAM_TAG_VALUE:
+		if (helper->item_temp == NULL) {
+			g_set_error_literal (error,
+					     APPSTREAM_CACHE_ERROR,
+					     APPSTREAM_CACHE_ERROR_FAILED,
+					     "item_temp mimetype invalid");
+			return;
+		}
+		appstream_app_add_metadata (helper->item_temp,
+					    helper->lang_temp,
+					    text,
+					    text_len);
+		g_free (helper->lang_temp);
+		helper->lang_temp = NULL;
+		break;
 	case APPSTREAM_TAG_COMPULSORY_FOR_DESKTOP:
 		if (helper->item_temp == NULL) {
 			g_set_error_literal (error,
@@ -733,16 +821,19 @@ appstream_cache_text_cb (GMarkupParseContext *context,
 				       helper->url_type_temp,
 				       text, text_len);
 		break;
-	case APPSTREAM_TAG_LICENCE:
-		if (helper->item_temp == NULL ||
-		    appstream_app_get_licence (helper->item_temp) != NULL) {
+	case APPSTREAM_TAG_PROJECT_LICENSE:
+		if (helper->item_temp == NULL) {
 			g_set_error_literal (error,
 					     APPSTREAM_CACHE_ERROR,
 					     APPSTREAM_CACHE_ERROR_FAILED,
-					     "item_temp licence invalid");
+					     "item_temp license invalid");
 			return;
 		}
-		appstream_app_set_licence (helper->item_temp, text, text_len);
+		if (appstream_app_get_project_license (helper->item_temp) == NULL) {
+			appstream_app_set_project_license (helper->item_temp,
+							   text,
+							   text_len);
+		}
 		break;
 	case APPSTREAM_TAG_DESCRIPTION:
 		appstream_markup_add_content (helper->markup, text, text_len);
@@ -781,6 +872,19 @@ appstream_cache_text_cb (GMarkupParseContext *context,
 						  text,
 						  text_len);
 		break;
+	case APPSTREAM_TAG_LANG:
+		if (helper->item_temp == NULL) {
+			g_set_error_literal (error,
+					     APPSTREAM_CACHE_ERROR,
+					     APPSTREAM_CACHE_ERROR_FAILED,
+					     "lang not started");
+			return;
+		}
+		appstream_app_add_locale (helper->item_temp,
+					  text,
+					  text_len,
+					  helper->priority);
+		break;
 	default:
 		/* ignore unknown entries */
 		break;
