@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2013 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2014 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -21,6 +21,7 @@
 
 #include <config.h>
 #include <glib/gi18n.h>
+#include <locale.h>
 
 #include <gs-plugin.h>
 #include <gs-plugin-loader.h>
@@ -33,6 +34,7 @@
 struct GsPluginPrivate {
 	AppstreamCache		*cache;
 	GPtrArray		*file_monitors;
+	gchar			*locale;
 	gchar			*cachedir;
 	gsize			 done_init;
 };
@@ -305,6 +307,7 @@ void
 gs_plugin_destroy (GsPlugin *plugin)
 {
 	g_free (plugin->priv->cachedir);
+	g_free (plugin->priv->locale);
 	g_object_unref (plugin->priv->cache);
 	g_ptr_array_unref (plugin->priv->file_monitors);
 }
@@ -318,8 +321,15 @@ gs_plugin_startup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 	AppstreamApp *app;
 	GPtrArray *items;
 	gboolean ret;
+	gchar *tmp;
 	guint i;
 
+	/* get the locale without the UTF-8 suffix */
+	plugin->priv->locale = g_strdup (setlocale (LC_MESSAGES, NULL));
+	tmp = g_strstr_len (plugin->priv->locale, -1, ".UTF-8");
+	if (tmp != NULL)
+		*tmp = '\0';
+
 	/* clear all existing file monitors */
 	g_ptr_array_set_size (plugin->priv->file_monitors, 0);
 
@@ -476,6 +486,7 @@ gs_plugin_refine_add_screenshots (GsApp *app, AppstreamApp *item)
 		return;
 
 	/* does the app already have some */
+	gs_app_add_kudo (app, GS_APP_KUDO_HAS_SCREENSHOTS);
 	if (gs_app_get_screenshots(app)->len > 0)
 		return;
 
@@ -565,13 +576,15 @@ gs_plugin_refine_item (GsPlugin *plugin,
 	}
 
 	/* set licence */
-	if (appstream_app_get_licence (item) != NULL && gs_app_get_licence (app) == NULL)
-		gs_app_set_licence (app, appstream_app_get_licence (item));
+	if (appstream_app_get_project_license (item) != NULL && gs_app_get_licence (app) == NULL)
+		gs_app_set_licence (app, appstream_app_get_project_license (item));
 
 	/* set keywords */
 	if (appstream_app_get_keywords (item) != NULL &&
-	    gs_app_get_keywords (app) == NULL)
+	    gs_app_get_keywords (app) == NULL) {
 		gs_app_set_keywords (app, appstream_app_get_keywords (item));
+		gs_app_add_kudo (app, GS_APP_KUDO_HAS_KEYWORDS);
+	}
 
 	/* set description */
 	if (appstream_app_get_description (item) != NULL) {
@@ -585,7 +598,8 @@ gs_plugin_refine_item (GsPlugin *plugin,
 		gs_plugin_refine_item_pixbuf (plugin, app, item);
 
 	/* set categories */
-	if (appstream_app_get_categories (item) != NULL && gs_app_get_categories (app) == NULL)
+	if (appstream_app_get_categories (item) != NULL &&
+	    gs_app_get_categories (app)->len == 0)
 		gs_app_set_categories (app, appstream_app_get_categories (item));
 
 	/* set project group */
@@ -610,6 +624,22 @@ gs_plugin_refine_item (GsPlugin *plugin,
 	/* set screenshots */
 	gs_plugin_refine_add_screenshots (app, item);
 
+	/* add kudos */
+	if (appstream_app_has_locale (item, plugin->priv->locale) > 50)
+		gs_app_add_kudo (app, GS_APP_KUDO_MY_LANGUAGE);
+	if (appstream_app_get_metadata_item (item, "X-Kudo-GTK3") != NULL)
+		gs_app_add_kudo (app, GS_APP_KUDO_MODERN_TOOLKIT);
+	if (appstream_app_get_metadata_item (item, "X-Kudo-SearchProvider") != NULL)
+		gs_app_add_kudo (app, GS_APP_KUDO_SEARCH_PROVIDER);
+	if (appstream_app_get_metadata_item (item, "X-Kudo-InstallsUserDocs") != NULL)
+		gs_app_add_kudo (app, GS_APP_KUDO_INSTALLS_USER_DOCS);
+	if (appstream_app_get_metadata_item (item, "X-Kudo-UsesNotifications") != NULL)
+		gs_app_add_kudo (app, GS_APP_KUDO_USES_NOTIFICATIONS);
+	if (appstream_app_get_metadata_item (item, "X-Kudo-RecentRelease") != NULL)
+		gs_app_add_kudo (app, GS_APP_KUDO_RECENT_RELEASE);
+	if (appstream_app_get_metadata_item (item, "X-Kudo-UsesAppMenu") != NULL)
+		gs_app_add_kudo (app, GS_APP_KUDO_USES_APP_MENU);
+
 	return ret;
 }
 
