@@ -199,4 +199,33 @@ gs_plugin_add_category_apps (GsPlugin *plugin,
 	return TRUE;
 }
 
+/**
+ * gs_plugin_refine:
+ */
+gboolean
+gs_plugin_refine (GsPlugin *plugin,
+		  GList **list,
+		  GsPluginRefineFlags flags,
+		  GCancellable *cancellable,
+		  GError **error)
+{
+	const gchar *id;
+	GList *l;
+	GsApp *app;
+	guint i;
+
+	for (i = 0; i < G_N_ELEMENTS (featured); i++) {
+		for (l = *list; l != NULL; l = l->next) {
+			app = GS_APP (l->data);
+			id = gs_app_get_id_full (app);
+			if (g_strcmp0 (id, featured[i].app) != 0)
+				continue;
+			gs_app_add_kudo (app, GS_APP_KUDO_FEATURED_RECOMMENDED);
+		}
+	}
+
+	return TRUE;
+
+}
+
 /* vim: set noexpandtab: */
