@@ -59,12 +59,15 @@ gs_plugin_initialize (GsPlugin *plugin)
 }
 
 /**
- * gs_plugin_get_priority:
+ * gs_plugin_get_deps:
  */
-gdouble
-gs_plugin_get_priority (GsPlugin *plugin)
+const gchar **
+gs_plugin_get_deps (GsPlugin *plugin)
 {
-	return 1.1f;
+	static const gchar *deps[] = {
+		"appstream",	/* requires the app_id from the pkgname */
+		NULL };
+	return deps;
 }
 
 /**
@@ -202,7 +205,7 @@ out:
  */
 gboolean
 gs_plugin_refine (GsPlugin *plugin,
-		  GList *list,
+		  GList **list,
 		  GsPluginRefineFlags flags,
 		  GCancellable *cancellable,
 		  GError **error)
@@ -226,7 +229,7 @@ gs_plugin_refine (GsPlugin *plugin,
 	}
 
 	/* add any missing ratings data */
-	for (l = list; l != NULL; l = l->next) {
+	for (l = *list; l != NULL; l = l->next) {
 		app = GS_APP (l->data);
 		if (gs_app_get_id (app) == NULL)
 			continue;
@@ -235,6 +238,7 @@ gs_plugin_refine (GsPlugin *plugin,
 		rating = gs_plugin_local_find_app (plugin, gs_app_get_id (app));
 		if (rating != -1) {
 			gs_app_set_rating (app, rating);
+			gs_app_set_rating_confidence (app, 100);
 			gs_app_set_rating_kind (app, GS_APP_RATING_KIND_USER);
 		}
 	}
