@@ -183,9 +183,11 @@ gs_plugin_packagekit_resolve_packages_app (GsPlugin *plugin,
 				gs_app_set_name (app,
 						 GS_APP_QUALITY_LOWEST,
 						 pk_package_get_name (package));
+#if PK_CHECK_VERSION(0,9,1)
 				gs_app_set_summary (app,
 						    GS_APP_QUALITY_LOWEST,
 						    pk_package_get_summary (package));
+#endif
 			}
 		}
 	}
@@ -497,6 +499,9 @@ gs_plugin_packagekit_refine_details_app (GsPlugin *plugin,
 			gs_app_set_description (app,
 						GS_APP_QUALITY_LOWEST,
 						desc);
+			gs_app_set_summary (app,
+					    GS_APP_QUALITY_LOWEST,
+					    pk_details_get_summary (details));
 			g_free (desc);
 			break;
 		}
