@@ -146,9 +146,12 @@ gs_plugin_packagekit_add_results (GsPlugin *plugin,
 		app = gs_app_new (NULL);
 		gs_app_add_source (app, pk_package_get_name (package));
 		gs_app_add_source_id (app, pk_package_get_id (package));
-		gs_app_set_metadata (app,
-				     "PackageKit::package-summary",
-				     pk_package_get_summary (package));
+		gs_app_set_name (app,
+				 GS_APP_QUALITY_LOWEST,
+				 pk_package_get_name (package));
+		gs_app_set_summary (app,
+				    GS_APP_QUALITY_LOWEST,
+				    pk_package_get_summary (package));
 		gs_app_set_management_plugin (app, "PackageKit");
 		gs_app_set_version (app, pk_package_get_version (package));
 		switch (pk_package_get_info (package)) {
